[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Exchange DataFeed Snapshot API - Symbol List client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://open.factset.com/products/factset-exchange-datafeed-snapshot-api/en-us)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.ExchangeDataFeedSnapshotAPISymbolList/0.21.15)](https://pypi.org/project/fds.sdk.ExchangeDataFeedSnapshotAPISymbolList/v/0.21.15)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet’s Exchange DataFeed Snapshot API provides cost-effective access to real-time and delayed global exchange data. Proprietary technology normalizes over 300 global exchanges and 150+ data fields. Asset types integrated include equities, futures, options, warrants, fixed income, mutual funds, ETFs, indices, commodities, and FX rates. <p>Cutting-edge technology ensures reliability and provides scalability that allow applications to request multiple items at a time. To simplify client-side development an entire response can be placed in a matrix or table for effortless integration into internal and external applications. Using specified output formats (CSV, XML, JSON) receive all standard fields by default or customize the list based on specific needs.</p></p>Below are the current hosts:</p><p>Production: api.factset.com<p>Sandbox: api-sandbox.factset.com</p>

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 0.21.15
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.ExchangeDataFeedSnapshotAPISymbolList==0.21.15
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.ExchangeDataFeedSnapshotAPISymbolList==0.21.15
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList
from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.api import snapshot_api
from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = snapshot_api.SnapshotApi(api_client)
    ids = "ids_example" # str | Requested symbols or securities. **Required** if not using `oc`. This is a comma-seperated list with a maximum of 500. Each symbol can be a FactSet Exchange Symbol, CUSIP, or SEDOL. This can be used in combination with `chain` parameter to pull in a custom portfolio.<P>**Try it out** - Choose one or multiple - IBM-USA, BABA-USA, WALMEX-MX, 7203-TKS, VOD-LON, NPN-JSE, MABAX, NZF, SPY-USA, AGG-USA, GLD-USA, AOR-USA, MNA-USA, UUP-USA, SP50-SPX, ESX-STX, XAO-ASX, WD-MSX, NG00-USA, GC00-USA, CC00-USA, C00-USA, FC00-USA, ER00-USA, EURUSD-FX1, USDMXN-FX1, AUDJPY-FX1, EURCZK-FX1, USDILS-FX1, USDZAR-FX1, US10YY-TU1, FDS#190621C00145000-USA, FDS#190621P00145000-USA. *Leave blank if you are trying the `oc` parameter*.</p> (optional)
    oc = "oc_example" # str | Option Chain, accepts one underlying symbol for which the option chain will be returned. **Required** if not using `ids`. <p>**Try it Out** - Only available for BABA-USA</p> (optional)
    fields = "fields_example" # str | All available fields. This is a comma-seperated list (no spaces). **Required** only when using `oc`. Otherwise, leave blank to return all fields by default. See definitions for details on available field names and Field ID #s. The Data Service Manual has complete reference.<p>**Try it Out** - All fields available. Use either FID # or name. For `oc`, recommend to use BID_1,ASK_1,STRIKE_PRICE. (optional)
    sf = "sf_example" # str | Symbol Filters, ability to apply filtering to the result of an option chain `oc` request based on available fields and values for the symbols. `sf`=VALUE_[filter],[field],[value]   * **VALUE_MINMAX** -  Field value will be within specified interval. ex. - VALUE_MINMAX,BID_1,min,max   * **VALUE_EQUALS** - Field value will equal specified value. ex. - VALUE_EQUALS,BID_1,value   * **VALUE_NOT_EQUALS** - Field value will not equal specified value. ex. - VALUE_NOT_EQUALS,BID_1,value      **Try it Out** - depends on fields used. Example - VALUE_MINMAX,STRIKE_PRICE,120,130   (optional)
    format = "XML" # str | The format of the output file.<p>**Try it Out** - All formats available</p> (optional) if omitted the server will use the default value of "XML"
    serv = "FDS1" # str | The Data Service that handles the request for the specified symbols. Available live services are -    * **FDS1** -Production Data Service   * **FDS_FUND** -Fundamental Data Service used for End-of-Day data.  **Try it Out** - use FDS1  (optional)
    req_id = "req_id_example" # str | Request Identification String. Can be used by the application to keep track of requests.  The id is not used by this service, however, it is included in the XML response.<p> **Try it Out** - any value can be passed through.</p> (optional)

    try:
        # Request a snapshot of exchange data for a list of symbols.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_df_snapshot(ids=ids, oc=oc, fields=fields, sf=sf, format=format, serv=serv, req_id=req_id)

        pprint(api_response)
    except fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.ApiException as e:
        print("Exception when calling SnapshotApi->get_df_snapshot: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Request a snapshot of exchange data for a list of symbols.
    #     api_response, http_status_code, response_headers = api_instance.get_df_snapshot_with_http_info(ids=ids, oc=oc, fields=fields, sf=sf, format=format, serv=serv, req_id=req_id)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.ApiException as e:
    #     print("Exception when calling SnapshotApi->get_df_snapshot: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Request a snapshot of exchange data for a list of symbols.
    #     async_result = api_instance.get_df_snapshot_async(ids=ids, oc=oc, fields=fields, sf=sf, format=format, serv=serv, req_id=req_id)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.ApiException as e:
    #     print("Exception when calling SnapshotApi->get_df_snapshot: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Request a snapshot of exchange data for a list of symbols.
    #     async_result = api_instance.get_df_snapshot_with_http_info_async(ids=ids, oc=oc, fields=fields, sf=sf, format=format, serv=serv, req_id=req_id)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.ApiException as e:
    #     print("Exception when calling SnapshotApi->get_df_snapshot: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList

configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList

configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList

configuration = fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api-sandbox.factset.com*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*SnapshotApi* | [**get_df_snapshot**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/SnapshotApi.md#get_df_snapshot) | **GET** /DFSnapshot | Request a snapshot of exchange data for a list of symbols.
*SnapshotApi* | [**post_df_snapshot**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/SnapshotApi.md#post_df_snapshot) | **POST** /DFSnapshot | Snap exchange data. Use if URL lengh of 2,048 is exceeded.


## Documentation For Models

 - [DFSnapshotResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/DFSnapshotResponse.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/ErrorResponseSubErrors.md)
 - [Fields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/Fields.md)
 - [Specialfield](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ExchangeDataFeedSnapshotAPISymbolList/v1/docs/Specialfield.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.apis and fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.api.default_api import DefaultApi`
- `from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.ExchangeDataFeedSnapshotAPISymbolList
from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.apis import *
from fds.sdk.ExchangeDataFeedSnapshotAPISymbolList.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

